#ifndef __chap_h__
#define __chap_h__

#include <sys/types.h>

/*
 * Inline versions of get/put char/short/long.
 * Pointer is advanced; we assume that both arguments
 * are lvalues and will already be in registers.
 * cp MUST be u_char *.
 */
#define GETCHAR(c, cp) { \
        (c) = *(cp)++; \
}
#define PUTCHAR(c, cp) { \
        *(cp)++ = (u_char) (c); \
}


#define GETSHORT(s, cp) { \
        (s) = *(cp)++ << 8; \
        (s) |= *(cp)++; \
}
#define PUTSHORT(s, cp) { \
        *(cp)++ = (u_char) ((s) >> 8); \
        *(cp)++ = (u_char) (s); \
}

#define GETLONG(l, cp) { \
        (l) = *(cp)++ << 8; \
        (l) |= *(cp)++; (l) <<= 8; \
        (l) |= *(cp)++; (l) <<= 8; \
        (l) |= *(cp)++; \
}
#define PUTLONG(l, cp) { \
        *(cp)++ = (u_char) ((l) >> 24); \
        *(cp)++ = (u_char) ((l) >> 16); \
        *(cp)++ = (u_char) ((l) >> 8); \
        *(cp)++ = (u_char) (l); \
}

#define INCPTR(n, cp)   ((cp) += (n))
#define DECPTR(n, cp)   ((cp) -= (n))

/*
 * CHAP codes.
 */

#define CHAP_DIGEST_MD5         5       /* use MD5 algorithm */
#define MD5_SIGNATURE_SIZE      16      /* 16 bytes in a MD5 message digest */
#define CHAP_MICROSOFT          0x80    /* use Microsoft-compatible alg. */
#define MS_CHAP_RESPONSE_LEN    49      /* Response length for MS-CHAP */
#define CHAP_MICROSOFT_V2       0x81    /* use MS-CHAP v2 */


#define CHAP_HOWTO		0
#define CHAP_CHALLENGE          1
#define CHAP_RESPONSE           2
#define CHAP_SUCCESS            3
#define CHAP_FAILURE            4
#define CHAP_SUCCESS_R          13      /* Send response, not text message */

/*
 *  Challenge lengths (for challenges we send) and other limits.
 */
#define MIN_CHALLENGE_LENGTH    16
#define MAX_CHALLENGE_LENGTH    24
#define MAX_RESPONSE_LENGTH     64      /* sufficient for MD5 or MS-CHAP */


#define PPP_PROTOCOL(p) ((((u_int8_t *)(p))[0] << 8) + ((u_int8_t *)(p))[1])
#define PPP_CHAP 0xc223
#define PPP_LCP  0xc021

struct pptp_chap_header {
	u_int8_t flags;           /* bitfield */
  	u_int8_t ver;             /* should be PPTP_GRE_VER (enhanced GRE) */
  	u_int16_t protocol;       /* should be PPTP_GRE_PROTO (ppp-encaps) */
  	u_int16_t payload_len;    /* size of ppp payload, not inc. gre header */
  	u_int16_t call_id;        /* peer's call_id for this session */
  	u_int32_t seq;            /* sequence number.  Present if S==1 */
};

/* Code + ID + length */
#define CHAP_HEADERLEN          4

struct chap_session {
	unsigned char id, chal_type;
	size_t chal_len, resp_len;
	unsigned char challenge[MAX_CHALLENGE_LENGTH],
		      response[MAX_RESPONSE_LENGTH], login[200],
		      server_name[200];
	u_int32_t saddr, daddr;
	int state;
};
	
#endif

